/*
 * Copyright 1994 Steve Passe
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of AUTHOR (Steve Passe)
 * are not be used in advertising or publicity pertaining to distribution of
 * the software without specific, written prior permission.  AUTHORS make no
 * representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * 1996 Revision
 * AUTHORS:     Steve Passe <smp@csn.net>
 */


static char rcsid[] = "$Id: focus.c,v 1.1 1994/12/06 06:35:51 smp Exp smp $";

#include <stdio.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <unistd.h>

#include "focus.h"

#define DEFAULT_PORT		"/dev/tty09"
#define DEFAULT_BAUD_VAL	B9600

#define FOCUS_READ			0x00
#define FOCUS_WRITE			0x80

#define FOCUS_KB			0x00
#define FOCUS_MOUSE			0x08

#define FOCUS_RMASK			0x3f
#define FOCUS_RVAL			0x80
#define FOCUS_KB_FIELD		0x07
#define FOCUS_MS_FIELD		0x38
#define FOCUS_MS_SHIFT		3

#define FOCUS_MIN			0
#define FOCUS_MAX			7


struct	termios	oldTp;
struct	termios	newTp;

char	*ofile_name = DEFAULT_PORT;

int		baudValue = DEFAULT_BAUD_VAL;
int		verbose = 0;

static char **doArgs( int, char ** );
static void	usage( char	* );
static int portInit( FILE * );
static int portRestore( FILE * );

static FILE*	fpout;


int
focusOpen( void )
{
	FILE*	fp;
	struct	termios	term;

	/* open file ( serial port ) */
	if ( (fp = fopen( ofile_name, "r+" )) == 0 ) return ERROR;

	/* read current attributes */
	if ( tcgetattr( fileno( fp ), &term ) == ERROR ) return ERROR;

	/* modify attributes */
	term.c_oflag &= ~( OPOST );
	term.c_cflag &= ~( CSIZE | CSTOPB | HUPCL | PARENB );
	term.c_cflag |= ( CLOCAL | CREAD | CS8 );
	term.c_lflag &= ~( ECHO | ECHOE | ECHOK | ECHONL |
						 ISIG | ICANON | NOFLSH | TOSTOP );

	/* set baud rates */
	if ( cfsetispeed( &term, baudValue ) == ERROR ) return ERROR;
	if ( cfsetospeed( &term, baudValue ) == ERROR ) return ERROR;

	/* set new attributes */
	if ( tcsetattr( fileno( fp ), TCSANOW, &term ) == ERROR ) return ERROR;

	return OK;
}


/*
 * set new values for keyboard & mouse focus in kb & mouse
 */
int
focusSet( int keyBoard, int mouse )
{
	int	newKeyBoard, newMouse;

	if ( (keyBoard < FOCUS_MIN) || (keyBoard > FOCUS_MAX)
			|| (mouse < FOCUS_MIN) || (mouse > FOCUS_MAX) ) return ERROR;

	fputc( FOCUS_WRITE | FOCUS_KB | keyBoard, fpout );
	focusGet( &newKeyBoard, NULL, 0 );
	if ( newKeyBoard != keyBoard ) return ERROR;

	fputc( FOCUS_WRITE | FOCUS_MOUSE | mouse, fpout );
	focusGet( NULL, &newMouse, 0 );
	if ( newMouse != mouse ) return ERROR;

	return OK;
}


/*
 * return the current values for keyboard & mouse focus in kb & mouse
 * flag == TRUE: request focusControl box to return status
 * flag == FALSE: expect status to be sent by previous focusSet() call
 */
int
focusGet( int* kb, int* mouse, int flag )
{
	int status;

	/* explicit request for current focus status */
	if ( flag ) fputc( FOCUS_READ, fpout );

	/* get the status from hardware */
	status = fgetc( fpout );

	/* look at 'signature' */
	if ( (status & ~FOCUS_RMASK) != FOCUS_RVAL ) return ERROR;

	/* set requested variables */
	if ( kb ) *kb = status & FOCUS_KB_FIELD;
	if ( mouse ) *mouse = (status & FOCUS_MS_FIELD) >> FOCUS_MS_SHIFT;

	return OK;
}


/***************************************************************************/

#if defined( MAIN )

static int
portInit( FILE *fp )
{
	if ( tcgetattr( fileno( fp ), &oldTp ) == ERROR ) return ERROR;
	newTp = oldTp;
	newTp.c_oflag &= ~( OPOST );
	newTp.c_cflag &= ~( CSIZE | CSTOPB | HUPCL | PARENB );
	newTp.c_cflag |= ( CLOCAL | CREAD | CS8 );
	newTp.c_lflag &= ~( ECHO | ECHOE | ECHOK | ECHONL |
						 ISIG | ICANON | NOFLSH | TOSTOP );
	if ( cfsetispeed( &newTp, baudValue ) == ERROR ) return ERROR;
	if ( cfsetospeed( &newTp, baudValue ) == ERROR ) return ERROR;

	if ( tcsetattr( fileno( fp ), TCSANOW, &newTp ) == ERROR ) return ERROR;

	return 0;
}


static int
portRestore( FILE *fp )
{
	if ( tcsetattr( fileno( fp ), TCSADRAIN, &oldTp ) == ERROR ) return ERROR;

	return 0;
}


static char **
doArgs( int	ac, char *av[] )
{
	char	*name;
	char 	*p;
	char	*baud_str;

	name = *av;									/* remember my name */
	while ( --ac ) {							/* for each arg */
		if ( *(p = *(++av)) == '-') {			/* if 1st char == '-' */
			switch( *(++p) & 0x7f ) {			/* act on next char */
case 'p':
				ofile_name = *(++av);
				break;

case 's':
				baud_str = *(++av);
				if ( strncmp( baud_str, "38400", 5 ) == 0 )
					baudValue = EXTB;
				else if ( strncmp( baud_str, "19200", 5 ) == 0 )
					baudValue = EXTA;
				else if ( strncmp( baud_str, "9600", 4 ) == 0 )
					baudValue = B9600;
				else
					goto foo;
				break;
case 'v':
				++verbose;
				break;
case '\0':										/* end of arg */
				/**
				 * we could put stdio redirection here...
				 * break;
				 */
default:
				goto foo;
			}
		}
		else break;
	}
	if ( ac == 0 ) av = '\0';			/* declare that we have no args left */
	return av;

foo:
	verbose = 1;
	usage( name );						/* bad arg */
	exit( 1 );
}


static void
usage( char	*name )
{
	if ( verbose ) {
		fprintf( stderr, "\n%s [-b][-i id][-p port][-s speed][-v] filename", name );
		fprintf( stderr, "\n\n  where:" );
		fprintf( stderr, "\n	-b enables binary downloads" );
		fprintf( stderr, "\n	-i sets the dprom id" );
		fprintf( stderr, "\n	-p sets the port" );
		fprintf( stderr, "\n	-s set the speed(baudrate)" );
		fprintf( stderr, "\n	-v increments the VERBOSE flag" );
		fprintf( stderr, "\n\n" );
	}
}


main( int argc, char *argv[] )
{
	int		c;
	char**	av;
	char**	doArgs();
	int keyBoard, mouse;
	int newKb, newMouse;
	av = doArgs( argc, argv );

	/* open file ( serial port ) */
	fpout = fopen( ofile_name, "r+" );
	if ( fpout == 0 ) {
		fprintf( stderr, "\ncouldn't open '%s' for output\n\n",
				ofile_name );
		exit( 1 );
	}
	if ( portInit( fpout ) == ERROR ) {
		fprintf( stderr, "\ncouldn't initialize '%s' for output\n\n",
				ofile_name );
		exit( 1 );
	}

	if ( verbose ) fprintf( stderr, "\t" );

	/* dump file */

#if 1

	srandom( 1 );
top:
	newKb = random() & 0x07;
	fputc( FOCUS_WRITE | FOCUS_KB | newKb, fpout );
	focusGet( &keyBoard, NULL, 0 );
	if ( verbose ) fprintf( stdout, "new kb: %d", keyBoard );
	sleep( 1 );
	newMouse = random() & 0x07;
	fputc( FOCUS_WRITE | FOCUS_MOUSE | newMouse, fpout );
	focusGet( NULL, &mouse, 0 );
	if ( verbose ) fprintf( stdout, ", new mouse: %d\n", mouse );
	sleep( 1 );

	goto top;

#else
	switch ( argc  )
	{
	case 2:
		newKb = newMouse = atoi( argv[ 1 ] );
		break;
	case 3:
		newKb = atoi( argv[ 1 ] );
		newMouse = atoi( argv[ 2 ] );
		break;
	default:
		newKb = newMouse = 0;
	}
	focusGet( &keyBoard, &mouse, 1 );
	fprintf( stdout, "old kb: %d, old mouse: %d\n", keyBoard, mouse );

	fputc( FOCUS_WRITE | FOCUS_KB | newKb, fpout );
	focusGet( &keyBoard, NULL, 0 );
	fprintf( stdout, "new kb: %d", keyBoard );
	fputc( FOCUS_WRITE | FOCUS_MOUSE | newMouse, fpout );
	focusGet( NULL, &mouse, 0 );
	fprintf( stdout, ", new mouse: %d\n", mouse );
#endif

	(void) portRestore ( fpout );

	if ( verbose ) fprintf( stderr, "\n" );
	exit( 0 );
}

#endif /* MAIN */


/* Local Variables: */
/* tab-width:4 */
/* c-indent-level:4 */
/* c-label-offset:-4 */
/* End: */
