/*
 * Copyright 1994 Steve Passe
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted, provided
 * that the above copyright notice appear in all copies and that both that
 * copyright notice and this permission notice appear in supporting
 * documentation, and that the name of AUTHOR (Steve Passe)
 * are not be used in advertising or publicity pertaining to distribution of
 * the software without specific, written prior permission.  AUTHORS make no
 * representations about the suitability of this software for any purpose.
 * It is provided "as is" without express or implied warranty.
 *
 * AUTHORS DISCLAIM ALL WARRANTIES WITH REGARD TO THIS SOFTWARE, INCLUDING ALL
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS, IN NO EVENT SHALL AUTHORS
 * BE LIABLE FOR ANY SPECIAL, INDIRECT OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES
 * WHATSOEVER RESULTING FROM LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION
 * OF CONTRACT, NEGLIGENCE OR OTHER TORTIOUS ACTION, ARISING OUT OF OR IN 
 * CONNECTION WITH THE USE OR PERFORMANCE OF THIS SOFTWARE.
 *
 * 1996 Revision
 * AUTHORS:     Steve Passe <smp@csn.net>
 */


static char rcsid[] = "$Id: focus.c,v 1.4 1995/02/03 20:16:24 smp Exp smp $";

#include <stdio.h>
#include <stdlib.h>
#include <fcntl.h>
#include <errno.h>
#include <signal.h>
#include <termios.h>
#include <unistd.h>

#include "focus.h"

typedef	void(*PFVV)(void);

#define FOCUS_READ		0x00
#define FOCUS_WRITE		0x80

#define FOCUS_KB		0x00
#define FOCUS_MOUSE		0x08

#define FOCUS_RMASK		0x3f
#define FOCUS_RVAL		0x80
#define FOCUS_KB_FIELD		0x07
#define FOCUS_MS_FIELD		0x38
#define FOCUS_MS_SHIFT		3

#define FOCUS_MIN		0
#define FOCUS_MAX		7


static FILE*	fpLog = NULL;
static FILE*	fpPort = NULL;


#define FA_NAME_CNT	8
#define FA_NAME_LEN	32

typedef struct {
    char	name[ FA_NAME_LEN+1 ];
    int		focus;
} focusRecord;


#if defined( CAN_READ_FOCUS_ASSOC )
/** we should read this from a file!!! */
static focusRecord	focusRecords[ FA_NAME_CNT ];
#else
static focusRecord	focusRecords[] =
{
    { "off",	0 },

    { "larry",	1 },
    { "moe",	2 },
    { "shemp",	3 },

    { "one",	1 },
    { "two",	2 },
    { "three",	3 },
    { "four",	4 },
    { "five",	5 },
    { "six",	6 },
    { "seven",	7 },

    { "END",	-1 }
};
#endif

static int	getFocusFromName( char* );
static int	readFocusAssoc( void );

int		verbose = 0;


int
focusOpen( char* focusLog, char* focusPort, int focusBaud, char* str )
{
    struct termios	term;

    /** parse whatever */
    if ( focusLog )
    {
	if ( (fpLog = fopen( focusLog, "a" )) == 0 )
	{
	      fprintf( stderr, "focusOpen(): can't open log: %s\n",
		      focusPort );
	      return -2;
	}
	atexit( (PFVV)focusClose );
    }

    /* open file ( serial port ) */
    if ( (fpPort = fopen( focusPort, "r+" )) == 0 )
    {
	fprintf( stderr, "focusOpen(): can't open focus port: %s\n",
		focusPort );
	return -3;
    }

    /* read current attributes */
    if ( tcgetattr( fileno( fpPort ), &term ) == ERROR ) return -4;

    /* modify attributes */
    term.c_oflag &= ~( OPOST );
    term.c_cflag &= ~( CSIZE | CSTOPB | HUPCL | PARENB );
    term.c_cflag |= ( CLOCAL | CREAD | CS8 );
    term.c_lflag &= ~( ECHO | ECHOE | ECHOK | ECHONL |
		      ISIG | ICANON | NOFLSH | TOSTOP );

    /* set baud rates */
    if ( cfsetispeed( &term, focusBaud ) == ERROR ) return -5;
    if ( cfsetospeed( &term, focusBaud ) == ERROR ) return -6;

    /* set new attributes */
    if ( tcsetattr( fileno( fpPort ), TCSANOW, &term ) == ERROR ) return -7;

    /* read the association file */
    if ( readFocusAssoc() )
    {
	fprintf( stderr, "focusOpen(): can't read association file: %s\n",
		FOCUS_ASSOC );
	return -8;
    }

    if ( fpLog )
    {
	fprintf( fpLog, "focusOpen():  pid: %d", getpid() );
	if ( str ) fprintf( fpLog, " %s", str );
	fputc( '\n', fpLog );
	fflush( fpLog );
    }

    /* return success */
    return OK;
}


int
focusClose( void )
{
    if ( fpLog != NULL )
    {
	fprintf( fpLog, "focusClose(): pid: %d\n", getpid() );
	fflush( fpLog );

	if ( fclose( fpLog ) != 0 )
	{
	      perror( "focusClose()" );
	      return ERROR;
	}
    }

    return OK;
}


/**
 * read the association file
 */
static int
readFocusAssoc( void )
{
/** hard-coded for now, eventually wwe will read FOCUS_ASSOC */

    return OK;
}


/*
 * set new values for keyboard & mouse focus in kb & mouse
 */
int
focusSet( int keyBoard, int mouse )
{
    int	newKeyBoard, newMouse;

    if ( keyBoard != -1 )
    {
	if ( (keyBoard < FOCUS_MIN) || (keyBoard > FOCUS_MAX) ) return -2;
	fputc( FOCUS_WRITE | FOCUS_KB | keyBoard, fpPort );
	focusGet( &newKeyBoard, NULL, 0 );			/* flush */
    }

    if ( mouse != -1 )
    {
	if ( (mouse < FOCUS_MIN) || (mouse > FOCUS_MAX) ) return -3;
	fputc( FOCUS_WRITE | FOCUS_MOUSE | mouse, fpPort );
	focusGet( NULL, &newMouse, 0 );				/* flush */
    }

    /* check status */
    focusGet( &newKeyBoard, &newMouse, 1 );
    if ( ((keyBoard != -1) && (newKeyBoard != keyBoard))
	|| ((mouse != -1) && (newMouse != mouse)) )
    {
	fprintf( fpLog, "focusSet( %d, %d ): %d, %d\n",
		keyBoard, mouse, newKeyBoard, newMouse );
	fflush( fpLog );
	return -4;
    }

    return OK;
}


/*
 * set new values for keyboard & mouse focus in kb & mouse
 */
int
focusNSet( char* keyBoardName, char* mouseName )
{
    int	keyBoard, mouse;
    int	newKeyBoard, newMouse;

    if ( keyBoardName )
    {
	keyBoard = getFocusFromName( keyBoardName );
	if ( (keyBoard < FOCUS_MIN) || (keyBoard > FOCUS_MAX) ) return -2;
	fputc( FOCUS_WRITE | FOCUS_KB | keyBoard, fpPort );
	focusGet( &newKeyBoard, NULL, 0 );
	if ( newKeyBoard != keyBoard ) return -3;
    }

    if ( mouseName )
    {
	mouse = getFocusFromName( mouseName );
	if ( (mouse < FOCUS_MIN) || (mouse > FOCUS_MAX) ) return -4;
	fputc( FOCUS_WRITE | FOCUS_MOUSE | mouse, fpPort );
	focusGet( NULL, &newMouse, 0 );
	if ( newMouse != mouse ) return -5;
    }

    return OK;
}


static int
getFocusFromName( char* name )
{
    int		x = 0;

    while ( focusRecords[ x ].focus != -1 )
    {
	if ( strcmp( focusRecords[ x ].name, name ) == 0 )
	  return focusRecords[ x ].focus;
	++x;
    }

    return ERROR;
}


/*
 * return the current values for keyboard & mouse focus in kb & mouse
 * flag == TRUE: request focusControl box to return status
 * flag == FALSE: expect status to be sent by previous focusSet() call
 */
int
focusGet( int* kb, int* mouse, int flag )
{
    int status;

    /* explicit request for current focus status */
    if ( flag ) fputc( FOCUS_READ, fpPort );

    /* get the status from hardware */
    status = fgetc( fpPort );

    /* look at 'signature' */
    if ( (status & ~FOCUS_RMASK) != FOCUS_RVAL ) return ERROR;

    /* set requested variables */
    if ( kb ) *kb = status & FOCUS_KB_FIELD;
    if ( mouse ) *mouse = (status & FOCUS_MS_FIELD) >> FOCUS_MS_SHIFT;

    return OK;
}
